/****************************************************************************
** Meta object code from reading C++ file 'textrender.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "textrender.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textrender.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextRender_t {
    QByteArrayData data[22];
    char stringdata0[293];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextRender_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextRender_t qt_meta_stringdata_TextRender = {
    {
QT_MOC_LITERAL(0, 0, 10), // "TextRender"
QT_MOC_LITERAL(1, 11, 15), // "fontSizeChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 32), // "showBufferScrollIndicatorChanged"
QT_MOC_LITERAL(4, 61, 20), // "allowGesturesChanged"
QT_MOC_LITERAL(5, 82, 6), // "redraw"
QT_MOC_LITERAL(6, 89, 14), // "updateTermSize"
QT_MOC_LITERAL(7, 104, 10), // "mousePress"
QT_MOC_LITERAL(8, 115, 6), // "eventX"
QT_MOC_LITERAL(9, 122, 6), // "eventY"
QT_MOC_LITERAL(10, 129, 9), // "mouseMove"
QT_MOC_LITERAL(11, 139, 12), // "mouseRelease"
QT_MOC_LITERAL(12, 152, 16), // "handleScrollBack"
QT_MOC_LITERAL(13, 169, 5), // "reset"
QT_MOC_LITERAL(14, 175, 14), // "cursorPixelPos"
QT_MOC_LITERAL(15, 190, 15), // "cursorPixelSize"
QT_MOC_LITERAL(16, 206, 11), // "scrollToEnd"
QT_MOC_LITERAL(17, 218, 9), // "fontWidth"
QT_MOC_LITERAL(18, 228, 10), // "fontHeight"
QT_MOC_LITERAL(19, 239, 13), // "fontPointSize"
QT_MOC_LITERAL(20, 253, 25), // "showBufferScrollIndicator"
QT_MOC_LITERAL(21, 279, 13) // "allowGestures"

    },
    "TextRender\0fontSizeChanged\0\0"
    "showBufferScrollIndicatorChanged\0"
    "allowGesturesChanged\0redraw\0updateTermSize\0"
    "mousePress\0eventX\0eventY\0mouseMove\0"
    "mouseRelease\0handleScrollBack\0reset\0"
    "cursorPixelPos\0cursorPixelSize\0"
    "scrollToEnd\0fontWidth\0fontHeight\0"
    "fontPointSize\0showBufferScrollIndicator\0"
    "allowGestures"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextRender[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       5,  100, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    0,   75,    2, 0x06 /* Public */,
       4,    0,   76,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   77,    2, 0x0a /* Public */,
       6,    0,   78,    2, 0x0a /* Public */,
       7,    2,   79,    2, 0x0a /* Public */,
      10,    2,   84,    2, 0x0a /* Public */,
      11,    2,   89,    2, 0x0a /* Public */,
      12,    1,   94,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      14,    0,   97,    2, 0x02 /* Public */,
      15,    0,   98,    2, 0x02 /* Public */,
      16,    0,   99,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,    8,    9,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,    8,    9,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,    8,    9,
    QMetaType::Void, QMetaType::Bool,   13,

 // methods: parameters
    QMetaType::QPoint,
    QMetaType::QSize,
    QMetaType::Void,

 // properties: name, type, flags
      17, QMetaType::Int, 0x00495001,
      18, QMetaType::Int, 0x00495001,
      19, QMetaType::Int, 0x00495103,
      20, QMetaType::Bool, 0x00495103,
      21, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       0,
       0,
       1,
       2,

       0        // eod
};

void TextRender::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextRender *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fontSizeChanged(); break;
        case 1: _t->showBufferScrollIndicatorChanged(); break;
        case 2: _t->allowGesturesChanged(); break;
        case 3: _t->redraw(); break;
        case 4: _t->updateTermSize(); break;
        case 5: _t->mousePress((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 6: _t->mouseMove((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 7: _t->mouseRelease((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 8: _t->handleScrollBack((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: { QPoint _r = _t->cursorPixelPos();
            if (_a[0]) *reinterpret_cast< QPoint*>(_a[0]) = std::move(_r); }  break;
        case 10: { QSize _r = _t->cursorPixelSize();
            if (_a[0]) *reinterpret_cast< QSize*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->scrollToEnd(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TextRender::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextRender::fontSizeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TextRender::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextRender::showBufferScrollIndicatorChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TextRender::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextRender::allowGesturesChanged)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<TextRender *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->fontWidth(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->fontHeight(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->fontPointSize(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->showBufferScrollIndicator(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->allowGestures(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<TextRender *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setFontPointSize(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setShowBufferScrollIndicator(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setAllowGestures(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TextRender::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPaintedItem::staticMetaObject>(),
    qt_meta_stringdata_TextRender.data,
    qt_meta_data_TextRender,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextRender::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextRender::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextRender.stringdata0))
        return static_cast<void*>(this);
    return QQuickPaintedItem::qt_metacast(_clname);
}

int TextRender::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPaintedItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void TextRender::fontSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TextRender::showBufferScrollIndicatorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextRender::allowGesturesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
