/****************************************************************************
** Meta object code from reading C++ file 'util.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "util.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'util.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Util_t {
    QByteArrayData data[55];
    char stringdata0[812];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Util_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Util_t qt_meta_stringdata_Util = {
    {
QT_MOC_LITERAL(0, 0, 4), // "Util"
QT_MOC_LITERAL(1, 5, 10), // "visualBell"
QT_MOC_LITERAL(2, 16, 0), // ""
QT_MOC_LITERAL(3, 17, 6), // "notify"
QT_MOC_LITERAL(4, 24, 3), // "msg"
QT_MOC_LITERAL(5, 28, 27), // "clipboardOrSelectionChanged"
QT_MOC_LITERAL(6, 56, 18), // "windowTitleChanged"
QT_MOC_LITERAL(7, 75, 24), // "windowOrientationChanged"
QT_MOC_LITERAL(8, 100, 15), // "fontSizeChanged"
QT_MOC_LITERAL(9, 116, 15), // "dragModeChanged"
QT_MOC_LITERAL(10, 132, 19), // "keyboardModeChanged"
QT_MOC_LITERAL(11, 152, 27), // "keyboardFadeOutDelayChanged"
QT_MOC_LITERAL(12, 180, 21), // "keyboardLayoutChanged"
QT_MOC_LITERAL(13, 202, 22), // "orientationModeChanged"
QT_MOC_LITERAL(14, 225, 24), // "showWelcomeScreenChanged"
QT_MOC_LITERAL(15, 250, 25), // "contentOrientationChanged"
QT_MOC_LITERAL(16, 276, 21), // "Qt::ScreenOrientation"
QT_MOC_LITERAL(17, 298, 13), // "openNewWindow"
QT_MOC_LITERAL(18, 312, 13), // "versionString"
QT_MOC_LITERAL(19, 326, 10), // "configPath"
QT_MOC_LITERAL(20, 337, 16), // "keyPressFeedback"
QT_MOC_LITERAL(21, 354, 10), // "notifyText"
QT_MOC_LITERAL(22, 365, 4), // "text"
QT_MOC_LITERAL(23, 370, 19), // "copyTextToClipboard"
QT_MOC_LITERAL(24, 390, 3), // "str"
QT_MOC_LITERAL(25, 394, 11), // "windowTitle"
QT_MOC_LITERAL(26, 406, 17), // "windowOrientation"
QT_MOC_LITERAL(27, 424, 8), // "canPaste"
QT_MOC_LITERAL(28, 433, 20), // "terminalHasSelection"
QT_MOC_LITERAL(29, 454, 10), // "fontFamily"
QT_MOC_LITERAL(30, 465, 10), // "uiFontSize"
QT_MOC_LITERAL(31, 476, 8), // "fontSize"
QT_MOC_LITERAL(32, 485, 8), // "dragMode"
QT_MOC_LITERAL(33, 494, 12), // "keyboardMode"
QT_MOC_LITERAL(34, 507, 20), // "keyboardFadeOutDelay"
QT_MOC_LITERAL(35, 528, 14), // "keyboardLayout"
QT_MOC_LITERAL(36, 543, 20), // "extraLinesFromCursor"
QT_MOC_LITERAL(37, 564, 7), // "charset"
QT_MOC_LITERAL(38, 572, 15), // "keyboardMargins"
QT_MOC_LITERAL(39, 588, 15), // "orientationMode"
QT_MOC_LITERAL(40, 604, 17), // "showWelcomeScreen"
QT_MOC_LITERAL(41, 622, 12), // "KeyboardMode"
QT_MOC_LITERAL(42, 635, 11), // "KeyboardOff"
QT_MOC_LITERAL(43, 647, 13), // "KeyboardFixed"
QT_MOC_LITERAL(44, 661, 12), // "KeyboardFade"
QT_MOC_LITERAL(45, 674, 12), // "KeyboardMove"
QT_MOC_LITERAL(46, 687, 8), // "DragMode"
QT_MOC_LITERAL(47, 696, 7), // "DragOff"
QT_MOC_LITERAL(48, 704, 12), // "DragGestures"
QT_MOC_LITERAL(49, 717, 10), // "DragScroll"
QT_MOC_LITERAL(50, 728, 10), // "DragSelect"
QT_MOC_LITERAL(51, 739, 15), // "OrientationMode"
QT_MOC_LITERAL(52, 755, 15), // "OrientationAuto"
QT_MOC_LITERAL(53, 771, 20), // "OrientationLandscape"
QT_MOC_LITERAL(54, 792, 19) // "OrientationPortrait"

    },
    "Util\0visualBell\0\0notify\0msg\0"
    "clipboardOrSelectionChanged\0"
    "windowTitleChanged\0windowOrientationChanged\0"
    "fontSizeChanged\0dragModeChanged\0"
    "keyboardModeChanged\0keyboardFadeOutDelayChanged\0"
    "keyboardLayoutChanged\0orientationModeChanged\0"
    "showWelcomeScreenChanged\0"
    "contentOrientationChanged\0"
    "Qt::ScreenOrientation\0openNewWindow\0"
    "versionString\0configPath\0keyPressFeedback\0"
    "notifyText\0text\0copyTextToClipboard\0"
    "str\0windowTitle\0windowOrientation\0"
    "canPaste\0terminalHasSelection\0fontFamily\0"
    "uiFontSize\0fontSize\0dragMode\0keyboardMode\0"
    "keyboardFadeOutDelay\0keyboardLayout\0"
    "extraLinesFromCursor\0charset\0"
    "keyboardMargins\0orientationMode\0"
    "showWelcomeScreen\0KeyboardMode\0"
    "KeyboardOff\0KeyboardFixed\0KeyboardFade\0"
    "KeyboardMove\0DragMode\0DragOff\0"
    "DragGestures\0DragScroll\0DragSelect\0"
    "OrientationMode\0OrientationAuto\0"
    "OrientationLandscape\0OrientationPortrait"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Util[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
      16,  136, // properties
       3,  200, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  109,    2, 0x06 /* Public */,
       3,    1,  110,    2, 0x06 /* Public */,
       5,    0,  113,    2, 0x06 /* Public */,
       6,    0,  114,    2, 0x06 /* Public */,
       7,    0,  115,    2, 0x06 /* Public */,
       8,    0,  116,    2, 0x06 /* Public */,
       9,    0,  117,    2, 0x06 /* Public */,
      10,    0,  118,    2, 0x06 /* Public */,
      11,    0,  119,    2, 0x06 /* Public */,
      12,    0,  120,    2, 0x06 /* Public */,
      13,    0,  121,    2, 0x06 /* Public */,
      14,    0,  122,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    1,  123,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      17,    0,  126,    2, 0x02 /* Public */,
      18,    0,  127,    2, 0x02 /* Public */,
      19,    0,  128,    2, 0x02 /* Public */,
      20,    0,  129,    2, 0x02 /* Public */,
      21,    1,  130,    2, 0x02 /* Public */,
      23,    1,  133,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 16,    2,

 // methods: parameters
    QMetaType::Void,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   22,
    QMetaType::Void, QMetaType::QString,   24,

 // properties: name, type, flags
      25, QMetaType::QString, 0x00495001,
      26, QMetaType::Int, 0x00495103,
      27, QMetaType::Bool, 0x00495001,
      28, QMetaType::Bool, 0x00495001,
      29, QMetaType::QString, 0x00095401,
      30, QMetaType::Int, 0x00095401,
      31, QMetaType::Int, 0x00495103,
      32, QMetaType::Int, 0x00495103,
      33, QMetaType::Int, 0x00495103,
      34, QMetaType::Int, 0x00495103,
      35, QMetaType::QString, 0x00495103,
      36, QMetaType::Int, 0x00095401,
      37, QMetaType::QString, 0x00095401,
      38, QMetaType::Int, 0x00095401,
      39, QMetaType::Int, 0x00495103,
      40, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       3,
       4,
       2,
       2,
       0,
       0,
       5,
       6,
       7,
       8,
       9,
       0,
       0,
       0,
      10,
      11,

 // enums: name, alias, flags, count, data
      41,   41, 0x0,    4,  215,
      46,   46, 0x0,    4,  223,
      51,   51, 0x0,    3,  231,

 // enum data: key, value
      42, uint(Util::KeyboardOff),
      43, uint(Util::KeyboardFixed),
      44, uint(Util::KeyboardFade),
      45, uint(Util::KeyboardMove),
      47, uint(Util::DragOff),
      48, uint(Util::DragGestures),
      49, uint(Util::DragScroll),
      50, uint(Util::DragSelect),
      52, uint(Util::OrientationAuto),
      53, uint(Util::OrientationLandscape),
      54, uint(Util::OrientationPortrait),

       0        // eod
};

void Util::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Util *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->visualBell(); break;
        case 1: _t->notify((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->clipboardOrSelectionChanged(); break;
        case 3: _t->windowTitleChanged(); break;
        case 4: _t->windowOrientationChanged(); break;
        case 5: _t->fontSizeChanged(); break;
        case 6: _t->dragModeChanged(); break;
        case 7: _t->keyboardModeChanged(); break;
        case 8: _t->keyboardFadeOutDelayChanged(); break;
        case 9: _t->keyboardLayoutChanged(); break;
        case 10: _t->orientationModeChanged(); break;
        case 11: _t->showWelcomeScreenChanged(); break;
        case 12: _t->contentOrientationChanged((*reinterpret_cast< Qt::ScreenOrientation(*)>(_a[1]))); break;
        case 13: _t->openNewWindow(); break;
        case 14: { QString _r = _t->versionString();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 15: { QString _r = _t->configPath();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->keyPressFeedback(); break;
        case 17: _t->notifyText((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 18: _t->copyTextToClipboard((*reinterpret_cast< QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::visualBell)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Util::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::notify)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::clipboardOrSelectionChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::windowTitleChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::windowOrientationChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::fontSizeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::dragModeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::keyboardModeChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::keyboardFadeOutDelayChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::keyboardLayoutChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::orientationModeChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Util::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Util::showWelcomeScreenChanged)) {
                *result = 11;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Util *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->windowTitle(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->windowOrientation(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->canPaste(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->terminalHasSelection(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->fontFamily(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->uiFontSize(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->fontSize(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->dragMode(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->keyboardMode(); break;
        case 9: *reinterpret_cast< int*>(_v) = _t->keyboardFadeOutDelay(); break;
        case 10: *reinterpret_cast< QString*>(_v) = _t->keyboardLayout(); break;
        case 11: *reinterpret_cast< int*>(_v) = _t->extraLinesFromCursor(); break;
        case 12: *reinterpret_cast< QString*>(_v) = _t->charset(); break;
        case 13: *reinterpret_cast< int*>(_v) = _t->keyboardMargins(); break;
        case 14: *reinterpret_cast< int*>(_v) = _t->orientationMode(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->showWelcomeScreen(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Util *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setWindowOrientation(*reinterpret_cast< int*>(_v)); break;
        case 6: _t->setFontSize(*reinterpret_cast< int*>(_v)); break;
        case 7: _t->setDragMode(*reinterpret_cast< int*>(_v)); break;
        case 8: _t->setKeyboardMode(*reinterpret_cast< int*>(_v)); break;
        case 9: _t->setKeyboardFadeOutDelay(*reinterpret_cast< int*>(_v)); break;
        case 10: _t->setKeyboardLayout(*reinterpret_cast< QString*>(_v)); break;
        case 14: _t->setOrientationMode(*reinterpret_cast< int*>(_v)); break;
        case 15: _t->setShowWelcomeScreen(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Util::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Util.data,
    qt_meta_data_Util,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Util::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Util::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Util.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Util::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 16;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Util::visualBell()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Util::notify(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Util::clipboardOrSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Util::windowTitleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Util::windowOrientationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Util::fontSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Util::dragModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Util::keyboardModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Util::keyboardFadeOutDelayChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Util::keyboardLayoutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Util::orientationModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Util::showWelcomeScreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
